package de.digisalt.tileEdit.model;

import java.awt.Color;
import java.io.File;
import java.util.Vector;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.dataContainers.AbstractPaletteContainer;
import de.digisalt.tileEdit.model.dataContainers.SingletonPaletteContainer;
import de.digisalt.tileEdit.model.dataContainers.TileContainer;

public class CurrentData {

	private AbstractPaletteContainer pal = SingletonPaletteContainer.getInstance(false);
	private int currentLeftColorNumber = 0;
	private int currentRightColorNumber = 1;
	private String currentFilePath = "";
	private String currentFileName = "";
	private String currentPalettePath = "";
	private String currentPaletteName = "";
	private static CurrentData self = null;
	private Vector<TileContainer> Tiles = new Vector<TileContainer>();
	private int firstVisibleTile = 0;
	private int numOfCols = 16;
	private boolean GridEnabled = true;
	
	
	private CurrentData()
	{}
	
	public static CurrentData getInstance()
	{
		if (self == null)
		{
			self = new CurrentData();
		}
		return self;	
	}
	
	public AbstractPaletteContainer getCurrentPalette()
	{
		return pal;
	}
	
	public int getCurrentLeftColorNumber()
	{
		return currentLeftColorNumber;
	}
	
	public int getCurrentRightColorNumber()
	{
		return currentRightColorNumber;
	}
	
	public void setCurrentLeftColorNumber(int val)
	{
		if (val >= 0 || val <= 15)
		{
			currentLeftColorNumber = val;			
		}
	}
	
	public void setCurrentRightColorNumber(int val)
	{
		if (val >= 0 || val <= 15)
		{
			currentRightColorNumber = val;			
		}
	}

	public void setLeftColorRedVal(int i) {
		try {
			Color col = pal.getColor(currentLeftColorNumber);
			int oldRed = Math.round(col.getRed()/8);
			if (oldRed < 0) oldRed = 0;
			if (oldRed > 31) oldRed = 31;
			int oldGreen = Math.round(col.getGreen()/8);
			if (oldGreen < 0) oldGreen = 0;
			if (oldGreen > 31) oldGreen = 31;
			int oldBlue = Math.round(col.getBlue()/8);
			if (oldBlue < 0) oldBlue = 0;
			if (oldBlue > 31) oldBlue = 31;
			pal.setColor(currentLeftColorNumber, i, oldGreen, oldBlue);
		} catch (Exception e) {
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting new red value"});
		}		
	}

	public void setLeftColorGreenVal(int i) {
		try {
			Color col = pal.getColor(currentLeftColorNumber);
			int oldRed = Math.round(col.getRed()/8);
			if (oldRed < 0) oldRed = 0;
			if (oldRed > 31) oldRed = 31;
			int oldGreen = Math.round(col.getGreen()/8);
			if (oldGreen < 0) oldGreen = 0;
			if (oldGreen > 31) oldGreen = 31;
			int oldBlue = Math.round(col.getBlue()/8);
			if (oldBlue < 0) oldBlue = 0;
			if (oldBlue > 31) oldBlue = 31;
			pal.setColor(currentLeftColorNumber, oldRed, i, oldBlue);
		} catch (Exception e) {
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting new green value"});
		}
	}

	public void setLeftColorBlueVal(int i) {
		try {
			Color col = pal.getColor(currentLeftColorNumber);
			int oldRed = Math.round(col.getRed()/8);
			if (oldRed < 0) oldRed = 0;
			if (oldRed > 31) oldRed = 31;
			int oldGreen = Math.round(col.getGreen()/8);
			if (oldGreen < 0) oldGreen = 0;
			if (oldGreen > 31) oldGreen = 31;
			int oldBlue = Math.round(col.getBlue()/8);
			if (oldBlue < 0) oldBlue = 0;
			if (oldBlue > 31) oldBlue = 31;
			pal.setColor(currentLeftColorNumber, oldRed, oldGreen, i);
		} catch (Exception e) {
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting new blue value"});
		}
	}
	
	public void setCurrentFileNameAndPath(String path)
	{
		if (path != null)
		{
			File f = new File(path);
			currentFileName = f.getName();
			currentFilePath = f.getAbsolutePath();			
		}
		else
		{
			currentFileName = "";
			currentFilePath = "";
		}
	}
	
	public void setCurrentPaletteNameAndPath(String path)
	{
		File f = new File(path);
		currentPaletteName = f.getName();
		currentPalettePath = f.getAbsolutePath();
	}
	
	public String getCurrentFileName()
	{
		return currentFileName;
	}
	
	public String getCurrentFilePath()
	{
		return currentFilePath;
	}
	
	public String getCurrentPaletteName()
	{
		return currentPaletteName;
	}
	
	public String getCurrentPalettePath()
	{
		return currentPalettePath;
	}

	public void removeAllTiles()
	{
		Tiles.clear();
	}
	
	public void addTile(TileContainer newTile)
	{
		Tiles.add(newTile);
	}
	
	public TileContainer getTileNumber(int x)
	{
		return Tiles.get(x);
	}
	
	public int getMaxTiles()
	{
		return Tiles.size();
	}
	
	public void setNumOfCols(int x)
	{
		numOfCols = x;
	}
	
	public int getNumOfCols()
	{
		return numOfCols;
	}
	
	public void setFirstDisplayedTile(int val)
	{
		firstVisibleTile = val;		
	}
	
	public int getFirstDisplayedTile()
	{
		return firstVisibleTile;
	}
	
	//um ein tile runter scrollen
	public void increaseFirstDisplayedTileByOne()
	{
		if (firstVisibleTile + 1 < Tiles.size()-(numOfCols*15))
		{
			firstVisibleTile++;
		}
	}
	
	//um 1 tile hoch scrollen
	public void decreaseFirstDisplayedTileByOne()
	{
		if (firstVisibleTile - 1 > -numOfCols)
		{
			firstVisibleTile--;
		}
	}
	
	//um eine reihe runter scrollen
	public void increaseFirstDisplayedTileByCol()
	{
		if (firstVisibleTile + numOfCols < Tiles.size()-(numOfCols*15))
		{
			firstVisibleTile = firstVisibleTile + numOfCols;			
		}
	}
	
	//um eine reihe hoch scrollen
	public void decreaseFirstDisplayedTileByCol()
	{
		if (firstVisibleTile - numOfCols > -numOfCols)
		{
			firstVisibleTile = firstVisibleTile - numOfCols;
		}
	}
	
	public boolean getGridEnabled()
	{
		return GridEnabled;
	}
	
	public void setGridEnabled(boolean val)
	{
		GridEnabled = val;
	}
}

