package de.digisalt.tileEdit.model;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.dataContainers.SingletonPaletteContainer;
import de.digisalt.tileEdit.model.dataContainers.TileContainer;
import de.digisalt.tileEdit.view.components.StatusBar;
import de.digisalt.tileEdit.view.windows.MainWindow;

public class Model {
	private static Model self = null;
	
	private Model()
	{
		
	}
	
	public static Model getInstace()
	{
		if (self == null)
		{
			self = new Model();
		}
		return self;
	}

	public boolean loadPaletteIntoEditor(String path) {
		InputStream in = null;
		try {
			in = new FileInputStream(path);
		} catch (FileNotFoundException e) {
			Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "Couldn't open palette "+path+"."});
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Couldn't open file "+path});
			return false;
		}
		
		int[] paletteData = new int[50];
		int temp1 = 0;
		int count53BytesRead = 1;
		for (long i = 0; i<=53; i++)
		{
			try {
				//in.read(temp1);
				temp1 = in.read();
			} catch (IOException e) {
				Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "Couldn't read palette "+path+"."});
				Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Couldn't read file "+path});
				return false;
			}
			//skip the first four bytes
			if (i<=3)
			{
				//do nothing with this byte
			}
			else
			{
				paletteData[count53BytesRead-5] = temp1;				
			}
			count53BytesRead++;
			if (count53BytesRead == 53)
			{
				//transform current content of colorInputArray to a real PaletteContaine
				for (int k=0; k<=45; k=k+3)
				{
					//get raw intValues
					int r = paletteData[k+0];
					int g = paletteData[k+1];
					int b = paletteData[k+2];
					//generate snes values (+1)
					r = r/8;
					g = g/8;
					b = b/8;
					paletteData[k+0] = r;
					paletteData[k+1] = g;
					paletteData[k+2] = b;
					//System.out.println("r|g|b: " + r + "|" + g + "|" + b);
				}
				//send paletteData to currently used palette
				SingletonPaletteContainer.getInstance(true).importColors(paletteData);
			}
		}
		Controller.getInstance().modelToView(Commands.m2vRepaintAllObjectsOnTileEditor, null);
		Controller.getInstance().viewToModel(Commands.v2mNewLeftButtonColorSelected, new Object[]{new Integer(0)});
		CurrentData.getInstance().setCurrentPaletteNameAndPath(path);
		Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_OK), "Opened palette "+path+" successfully."});
		return true;
	}

	public boolean loadFileIntoEditor(String path)
	{
		File input = new File(path);
		long length = input.length();
		InputStream in = null;
		try {
			in = new FileInputStream(path);
		} catch (FileNotFoundException e) {
			Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "Couldn't open file "+path+"."});
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Couldn't open file "+path});
			return false;
		}
		
		byte[] temp1 = new byte[] {0x0};
		BigInteger[] tileInputArray = new BigInteger[32];
		BigInteger byte1 = new BigInteger("0");
		BigInteger byte2 = new BigInteger("0");
		BigInteger byte17 = new BigInteger("0");
		BigInteger byte18 = new BigInteger("0");
		BigInteger outputBitString = new BigInteger("0");
		int[] tileData = new int[64];
		String binString = "";
		
		int count32BytesRead = 1;
		CurrentData.getInstance().removeAllTiles();
		for (long i = 0; i<=length; i++)
		{
			try {
				in.read(temp1);
			} catch (IOException e) {
				Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "Couldn't read file "+path+"."});
				Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Couldn't read file "+path});
				return false;
			}
			tileInputArray[count32BytesRead-1] = new BigInteger(temp1);
			count32BytesRead++;
			if (count32BytesRead == 33)
			{
				//transform current content of tileInputArray to a real TileContainer
				//this loop reads 8x8 pixels in sequence --> 32 bytes of input necessary --> 4 bit/pixel
				for (int k=0; k<=7; k++)
				{
					//always the first, second, seventeenth and eighteenth byte (lateron plus x times 2) are taken to get eight pixel values at a time
					//--> each of the four bytes contains one single bit for each colorIndex value.
					byte1 = tileInputArray[k*2];
					byte2 = tileInputArray[k*2 + 1];
					byte17 = tileInputArray[k*2 + 16];
					byte18 = tileInputArray[k*2 + 17];
					for (int l=0; l<=7; l++)
					{
						//binString is filled with 1s and 0s --> indicates the colorIndex to be used for this pixel
						binString = "";
						if (byte18.testBit(l)) binString = binString + "1"; else binString = binString + "0";
						if (byte17.testBit(l)) binString = binString + "1"; else binString = binString + "0";
						if (byte2.testBit(l))  binString = binString + "1";	else binString = binString + "0";
						if (byte1.testBit(l))  binString = binString + "1";	else binString = binString + "0";
						//creation of BigInt with the binary String to enable base transformation
						outputBitString = new BigInteger(binString, 2);
						//put the colorIndex value of currentPixel into array (decimal value)
						tileData[k*8 + l] = Integer.valueOf(outputBitString.toString(10));
					}
				}
				//create new TileContainer
				TileContainer t = new TileContainer(false);
				//import the just generated int-array as color indexes into the tileContainer
				t.importColorValues(tileData);
				
				//reset counter --> another 32 bytes have to be read if
				//another tileContainer should be generated...
				count32BytesRead=1;
				CurrentData.getInstance().addTile(t);
			}
		}
		CurrentData.getInstance().setFirstDisplayedTile(0);
		Controller.getInstance().modelToView(Commands.m2vUpdateTiles, null);
		MainWindow.getInstance().repaintTileSource();
		MainWindow.getInstance().repaint();
		CurrentData.getInstance().setCurrentFileNameAndPath(path);
		try {
			in.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
		Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_OK), "File "+path+" opened successfully."});
		return true;
	}
	
	public void newFile(int height, int width)
	{
		CurrentData.getInstance().removeAllTiles();
		CurrentData.getInstance().setFirstDisplayedTile(0);
		int num = height * width;
		for (int i=0; i<num; i++)
		{
			CurrentData.getInstance().addTile(new TileContainer(false));			
		}
		Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_NONE), "New file created with "+ width +" x "+ height+" tiles."});
	}

	public void saveDataToFile(String path) {
		//File out = new File(path);
		FileOutputStream out = null;
		try {
			out = new FileOutputStream(path);
		} catch (FileNotFoundException e1) {
			e1.printStackTrace();
			Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "File could not be saved."});
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[] {"File could not be saved.\n"+e1.getMessage()+"\nMake sure you have the permission to write into it."});
		}
		int max = CurrentData.getInstance().getMaxTiles();
		//iterate over all tiles registered in currentData vector
		for (int i=0; i<max; i++)
		{
			//get current Tile and transform it into a 32 byte long byte array
			TileContainer tile = CurrentData.getInstance().getTileNumber(i);
			//32 byte long byte array to be written into the file
			int[] oneTile = new int[32];
			for (int z=0; z<=31; z++)
			{
				oneTile[z] = 0;
			}
			for (int k=0; k<=7; k++)
			{
				int byte1 = oneTile[k*2];
				int byte2 = oneTile[k*2 + 1];
				int byte17 = oneTile[k*2 + 16];
				int byte18 = oneTile[k*2 + 17];
				for (int j=0; j<=7; j++)
				{
					try {
						BigInteger color = new BigInteger(""+tile.getColorAt(7-j, k));
						if (color.testBit(0))
						{
							byte1 = byte1 | (0x1 << j);
						}
						if (color.testBit(1))
						{
							byte2 = byte2 | (0x1 << j);
						}
						if (color.testBit(2))
						{
							byte17 = byte17 | (0x1 << j);
						}
						if (color.testBit(3))
						{
							byte18 = byte18 | (0x1 << j);
						}
						oneTile[k*2] = byte1;
						oneTile[k*2 + 1] = byte2;
						oneTile[k*2 + 16] = byte17;
						oneTile[k*2 + 17] = byte18;
					} catch (Exception e) {
						Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_WARN), "Couldn't read tile data..."});
						e.printStackTrace();
					}					
				}
			}
			for (int f=0; f<oneTile.length; f++)
			{
				byte[] b = new byte[1];
				b[0] = (byte)oneTile[f];
				try {
					out.write(b);
				} catch (IOException e) {
					Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "File could not be saved."});
					Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[] {"File could not be saved.\n"+e.getMessage()+"\nMake sure you have the permission to write into it."});
					e.printStackTrace();
				}				
			}
		}
		try {
			out.close();
		} catch (IOException e) {
			Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_ERROR), "File could not be saved."});
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[] {"File could not be saved.\n"+e.getMessage()+"\nMake sure you have the permission to write into it."});
			e.printStackTrace();
		}
		Controller.getInstance().modelToView(Commands.m2vShowStatusMessage, new Object[] {new Integer(StatusBar.STATUS_OK), "File saved at "+path+" successfully."});
	}
	
	
}
