package de.digisalt.tileEdit.view.components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.CurrentData;
import de.digisalt.tileEdit.model.dataContainers.AbstractPaletteContainer;

@SuppressWarnings("serial")
public class PalettePanel extends JPanel {
	private AbstractPaletteContainer pal = null;
	
	public PalettePanel()
	{
		pal = CurrentData.getInstance().getCurrentPalette();	
		this.addMouseListener(new MouseListener()
		{
			public void mouseClicked(MouseEvent arg0) {}
			public void mouseEntered(MouseEvent arg0) {}
			public void mouseExited(MouseEvent arg0) {}
			public void mousePressed(MouseEvent arg0) {}
			public void mouseReleased(MouseEvent arg0) 
			{
				int x = arg0.getX();
				int y = arg0.getY();
				x = x - 8;
				y = y - 12;
				int row = -1;
				if (y >= 0  && y <= 36) row = 1;
				if (y >= 37 && y <= 72) row = 2;
				int col = -1;
				col = x / 20;
				int colorNum = -1;
				if (row != -1 && col >= 0 && col <= 7)
				{
					if (row == 1) colorNum = col;
					if (row == 2) colorNum = 8 + col;
				}
				if (colorNum != -1)
				{
					//get Button that was pressed:
					if (arg0.getButton() == MouseEvent.BUTTON1)
					{
						//trigger event to controller --> select color number x with left button
						Controller.getInstance().viewToModel(Commands.v2mNewLeftButtonColorSelected, new Object[]{new Integer(colorNum)});
					}
					else if (arg0.getButton() == MouseEvent.BUTTON3)
					{
						//trigger event to controller --> select color number x with right button
						Controller.getInstance().viewToModel(Commands.v2mNewRightButtonColorSelected, new Object[]{new Integer(colorNum)});
					}
				}
				
				//x = 0 - 160
				//y = 0 - 72
			}
		});
	}
	
	public PalettePanel(AbstractPaletteContainer pal)
	{
		this.pal = pal;
	}

	public Color getColorNumber(int i) throws Exception
	{
		if (i < 0 || i > 15)
		{
			throw new Exception();
		}
		return pal.getColor(i);
	}
	
	public void setColorNumber(int i, int r, int g, int b) throws Exception
	{
		if (i < 0 || i > 15)
		{
			throw new Exception();
		}
		pal.setColor(i, r, g, b);
	}
	
	public void setPalette(AbstractPaletteContainer pal)
	{
		this.pal = pal;
	}
	
	public void paintComponent(Graphics g) 
	{
		super.paintComponent(g);       
		
		try 
		{
			//Draw Upper Row
			g.setColor(pal.getColor(0));
			g.fillRect(8, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(8, 12, 20, 36);
			g.setColor(pal.getColor(1));
			g.fillRect(28, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(28, 12, 20, 36);
			g.setColor(pal.getColor(2));
			g.fillRect(48, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(48, 12, 20, 36);
			g.setColor(pal.getColor(3));
			g.fillRect(68, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(68, 12, 20, 36);
			g.setColor(pal.getColor(4));
			g.fillRect(88, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(88, 12, 20, 36);
			g.setColor(pal.getColor(5));
			g.fillRect(108, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(108, 12, 20, 36);
			g.setColor(pal.getColor(6));
			g.fillRect(128, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(128, 12, 20, 36);
			g.setColor(pal.getColor(7));
			g.fillRect(148, 12, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(148, 12, 20, 36);
			//Draw Lower Row
			g.setColor(pal.getColor(8));
			g.fillRect(8, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(8, 48, 20, 36);
			g.setColor(pal.getColor(9));
			g.fillRect(28, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(28, 48, 20, 36);
			g.setColor(pal.getColor(10));
			g.fillRect(48, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(48, 48, 20, 36);
			g.setColor(pal.getColor(11));
			g.fillRect(68, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(68, 48, 20, 36);
			g.setColor(pal.getColor(12));
			g.fillRect(88, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(88, 48, 20, 36);
			g.setColor(pal.getColor(13));
			g.fillRect(108, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(108, 48, 20, 36);
			g.setColor(pal.getColor(14));
			g.fillRect(128, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(128, 48, 20, 36);
			g.setColor(pal.getColor(15));
			g.fillRect(148, 48, 20, 36);
			g.setColor(new Color(0,0,0));
			g.drawRect(148, 48, 20, 36);
		} 
		catch (Exception e) 
		{
			Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error painting palette panel"});
		}
	}
}
