package de.digisalt.tileEdit.view.components;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.TransferHandler;

import de.digisalt.tileEdit.controller.Commands;
import de.digisalt.tileEdit.controller.Controller;
import de.digisalt.tileEdit.model.CurrentData;
import de.digisalt.tileEdit.model.dataContainers.AbstractPaletteContainer;
import de.digisalt.tileEdit.model.dataContainers.TileContainer;

@SuppressWarnings("serial")
public class TilePanel4x extends JPanel 
{
	private AbstractPaletteContainer pal = null;
	private boolean containsData = false;
	private TileContainer data = null;
	private boolean isFilled = true;
	
	public TilePanel4x(AbstractPaletteContainer newPal)
	{
		data = new TileContainer(true);
		pal = newPal;
		this.setTransferHandler(new TransferHandler("data"));
		this.addMouseListener(new MouseAdapter() {
			public void mousePressed(MouseEvent e) 
			{
				JComponent c = (JComponent)e.getSource();
				TransferHandler th = c.getTransferHandler();
				th.exportAsDrag(c, e, TransferHandler.COPY);
		    }
		});
	}
	
	public TilePanel4x(AbstractPaletteContainer newPal, TileContainer t)
	{
		data = t;
		containsData = true;
		pal = newPal;
	}
	
	public TileContainer getData()
	{
		return data;
	}
	
	public void setData(TileContainer t)
	{
		data = t;
		containsData = true;
		pal = CurrentData.getInstance().getCurrentPalette();
		repaint();
	}
	
	public TileContainer getCopy()
	{
		if (!isFilled) return null;
		return data;
	}
	
	public void setCopy(TileContainer newData)
	{
		if (isFilled)
		{
			for (int i=0; i<=7; i++)
			{
				for (int j=0; j<=7; j++)
				{
					try {
						data.setColorAt(i, j, newData.getColorAt(i, j));
					} catch (Exception e) {
						Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error using drag and drop"});
						return;
					}				
				}
			}			
			repaint();
			Controller.getInstance().modelToView(Commands.m2vRepaintArrangerOnly, null);
		}
	}
	
	public void removeData()
	{
		data = new TileContainer(true);
		containsData = false;
		pal = CurrentData.getInstance().getCurrentPalette();
	}
	
	public void paintComponent(Graphics g) 
	{
        super.paintComponent(g);       

        // Draw Pixels
        for (int i = 0; i<=7; i++)
        {
        	for (int j = 0; j<=7; j++)
        	{
        		//getCurrentPixelColor
        		try {
					g.setColor(pal.getColor(data.getColorAt(i, j)));
				} catch (Exception e) {
					Controller.getInstance().modelToView(Commands.m2vShowAlert, new Object[]{"Error setting color to draw."});
				}
        		g.fillRect(i*4,j*4,4,4);
        	}
        }  
        //draw Gridlines if wanted
        if (CurrentData.getInstance().getGridEnabled())
        {
        	g.setColor(new Color(127,127,127));
        	g.drawRect(0, 0, 32, 32);
        }
        
    }  
	
	public boolean containsData()
	{
		return containsData;
	}
	
	public boolean getIsFilled()
	{
		return isFilled;
	}
	
	public void setIsFilled(boolean val)
	{
		isFilled = val;
	}
}
